@@ -22,13 +22,15 @@
 
 #include <config.h>
 #include <stdio.h>
-#include <sys/stat.h>   
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <string.h>
+#include <strings.h>
 #include <ctype.h>
+#include <time.h>
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
@@ -161,7 +163,7 @@ comment_data_dup (CommentData *data)
 		new_data->keywords = g_new0 (char*, data->keywords_n + 1);
 		new_data->keywords_n = data->keywords_n;
 
-		for (i = 0; i < data->keywords_n; i++) 
+		for (i = 0; i < data->keywords_n; i++)
 			new_data->keywords[i] = g_strdup (data->keywords[i]);
 		new_data->keywords[i] = NULL;
 	}
@@ -178,7 +180,7 @@ comment_data_dup (CommentData *data)
 
 
 gboolean
-comment_data_equal (CommentData *data1, 
+comment_data_equal (CommentData *data1,
 		    CommentData *data2)
 {
 	int i;
@@ -188,16 +190,16 @@ comment_data_equal (CommentData *data1,
 	if ((data1 == NULL) || (data2 == NULL))
 		return FALSE;
 
-	if (strcmp_null_tollerant (data1->place, data2->place) != 0)
+	if (strcmp_null_tolerant (data1->place, data2->place) != 0)
 		return FALSE;
 	if (data1->time != data2->time)
 		return FALSE;
-	if (strcmp_null_tollerant (data1->comment, data2->comment) != 0)
+	if (strcmp_null_tolerant (data1->comment, data2->comment) != 0)
 		return FALSE;
 	if (data1->keywords_n != data2->keywords_n)
 		return FALSE;
 	for (i = 0; i < data1->keywords_n; i++)
-		if (strcmp_null_tollerant (data1->keywords[i], 
+		if (strcmp_null_tolerant (data1->keywords[i],
 					   data2->keywords[i]) != 0)
 			return FALSE;
 
@@ -206,19 +208,18 @@ comment_data_equal (CommentData *data1,
 
 
 char *
-comments_get_comment_filename__old (const char *source,
-				    gboolean    resolve_symlinks,
-				    gboolean    unescape) 
+comments_get_comment_filename (const char *uri,
+			       gboolean    resolve_symlinks)
 {
-	char        *source_real = NULL;
-	char        *directory = NULL;
-	const char  *filename = NULL;
-	char        *path = NULL;
+	char *source_real = NULL;
+	char *directory = NULL;
+	char *filename = NULL;
+	char *comment_uri = NULL;
 
-	if (source == NULL) 
+	if (uri == NULL) 
 		return NULL;
 
-	source_real = g_strdup (source);
+	source_real = g_strdup (uri);
 
 	if (resolve_symlinks) {
 		char           *resolved = NULL;
@@ -226,123 +227,24 @@ comments_get_comment_filename__old (const char *source,
 
 		result = resolve_all_symlinks (source_real, &resolved);
 
-		if (result == GNOME_VFS_OK) { 
+		if (result == GNOME_VFS_OK) {
 			g_free (source_real);
 			source_real = resolved;
-		} else
-			g_free (resolved);
-	}
-
-	directory = remove_level_from_path (source_real);
-	filename = file_name_from_path (source_real);
-
-	path = g_strconcat (g_get_home_dir(), 
-			    "/", 
-			    RC_COMMENTS_DIR, 
-			    directory,
-			    "/",
-			    filename, 
-			    COMMENT_EXT, 
-			    NULL);
-
-	if (!unescape) {
-		char *escaped_path = escape_uri (path);
-		g_free (path);
-		path = escaped_path;
-	}
-
-	g_free (directory);
-	g_free (source_real);
-
-	return path;
-}
-
-
-char *
-comments_get_comment_dir__old (const char *directory,
-			       gboolean    resolve_symlinks,
-			       gboolean    unescape) 
-{
-	char        *directory_resolved = NULL;
-	const char  *directory_real = directory;
-	const char  *separator;
-	char        *path;
-
-	if (resolve_symlinks && (directory != NULL)) {
-		GnomeVFSResult  result;
-		result = resolve_all_symlinks (directory, &directory_resolved);
-		if (result == GNOME_VFS_OK) 
-			directory_real = directory_resolved;
-	}
-
-	if (directory_real == NULL)
-		separator = NULL;
-	else
-		separator = (directory_real[0] == '/') ? "" : "/";
-
-	path = g_strconcat (g_get_home_dir(), 
-			    "/", 
-			    RC_COMMENTS_DIR, 
-			    separator,
-			    directory_real,
-			    NULL);
-
-	g_free (directory_resolved);
-
-	if (unescape) {
-		char *unesc_path = gnome_vfs_unescape_string (path, NULL);
-		g_free (path);
-		path = unesc_path;
-	}
-
-	return path;
-}
-
-
-char *
-comments_get_comment_filename (const char *source,
-			       gboolean    resolve_symlinks,
-			       gboolean    unescape) 
-{
-	char  *source_real = NULL;
-	char  *directory = NULL;
-	char  *filename = NULL;
-	char  *path = NULL;
-
-	if (source == NULL) 
-		return NULL;
-
-	source_real = g_strdup (source);
-
-	if (resolve_symlinks) {
-		char           *resolved = NULL;
-		GnomeVFSResult  result;
-
-		result = resolve_all_symlinks (source_real, &resolved);
-
-		if (result == GNOME_VFS_OK) { 
-			g_free (source_real);
-			source_real = resolved;
-		} else
+		} 
+		else
 			g_free (resolved);
 	}
 
 	directory = remove_level_from_path (source_real);
 	filename = g_strconcat (file_name_from_path (source_real), COMMENT_EXT, NULL);
 
-	path = g_build_filename (directory, ".comments", filename, NULL);
-
-	if (!unescape) {
-		char *escaped_path = escape_uri (path);
-		g_free (path);
-		path = escaped_path;
-	}
+	comment_uri = g_strconcat (directory, "/.comments/", filename, NULL);
 
 	g_free (directory);
 	g_free (filename);
 	g_free (source_real);
 
-	return path;
+	return comment_uri;
 }
 
 
@@ -350,19 +252,26 @@ comments_get_comment_filename (const char *source,
 
 
 static CommentData *
-load_comment_from_iptc (const char *filename)
+load_comment_from_iptc (const char *uri)
 {
 	CommentData *data;
 	IptcData    *d;
 	struct tm    t;
 	int          i;
 	int          got_date = 0, got_time = 0;
+        char        *local_file = NULL;
 
-	if (filename == NULL)
+	if (uri == NULL)
 		return NULL;
 
-	d = iptc_data_new_from_jpeg (filename);
-	if (!d)
+        local_file = get_cache_filename_from_uri (uri);
+        if (local_file == NULL)
+                return NULL;
+
+	d = iptc_data_new_from_jpeg (local_file);
+	g_free (local_file);
+	
+	if (d == NULL) 
 		return NULL;
 
 	data = comment_data_new ();
@@ -373,7 +282,7 @@ load_comment_from_iptc (const char *filename)
 	for (i = 0; i < d->count; i++) {
 		IptcDataSet * ds = d->datasets[i];
 
-		if ((ds->record == IPTC_RECORD_APP_2) 
+		if ((ds->record == IPTC_RECORD_APP_2)
 		    && (ds->tag == IPTC_TAG_CAPTION)) {
 			if (data->comment)
 				continue;
@@ -381,7 +290,7 @@ load_comment_from_iptc (const char *filename)
 			if (data->comment)
 				iptc_dataset_get_data (ds, (guchar*)data->comment, ds->size + 1);
 		}
-		else if ((ds->record == IPTC_RECORD_APP_2) 
+		else if ((ds->record == IPTC_RECORD_APP_2)
 			 && (ds->tag == IPTC_TAG_CONTENT_LOC_NAME)) {
 			if (data->place)
 				continue;
@@ -396,7 +305,7 @@ load_comment_from_iptc (const char *filename)
 				continue;
 			comment_data_add_keyword (data, keyword);
 		}
-		else if ((ds->record == IPTC_RECORD_APP_2) 
+		else if ((ds->record == IPTC_RECORD_APP_2)
 			 && (ds->tag == IPTC_TAG_DATE_CREATED)) {
 			int year, month;
 			iptc_dataset_get_date (ds, &year, &month, &t.tm_mday);
@@ -404,7 +313,7 @@ load_comment_from_iptc (const char *filename)
 			t.tm_mon = month - 1;
 			got_date = 1;
 		}
-		else if ((ds->record == IPTC_RECORD_APP_2) 
+		else if ((ds->record == IPTC_RECORD_APP_2)
 			 && (ds->tag == IPTC_TAG_TIME_CREATED)) {
 			iptc_dataset_get_time (ds, &t.tm_hour, &t.tm_min, &t.tm_sec, NULL);
 			got_time = 1;
@@ -431,7 +340,7 @@ clear_iptc_comment (IptcData *d)
 
 	for (i = 0; deletetag[i] != 0; i++) {
 		IptcDataSet *ds;
-		while ((ds = iptc_data_get_dataset (d, 
+		while ((ds = iptc_data_get_dataset (d,
 						    IPTC_RECORD_APP_2,
 						    deletetag[i]))) {
 			iptc_data_remove_dataset (d, ds);
@@ -442,7 +351,7 @@ clear_iptc_comment (IptcData *d)
 
 
 static void
-save_iptc_data (const char *filename, 
+save_iptc_data (const char *filename,
 		IptcData   *d)
 {
 	guint        buf_len = 256 * 256;
@@ -476,8 +385,8 @@ save_iptc_data (const char *filename,
 	if (iptc_data_save (d, &iptc_buf, &iptc_len) < 0)
 		goto abort3;
 
-	ps3_len = iptc_jpeg_ps3_save_iptc (ps3_buf, ps3_len, 
-					   iptc_buf, iptc_len, 
+	ps3_len = iptc_jpeg_ps3_save_iptc (ps3_buf, ps3_len,
+					   iptc_buf, iptc_len,
 					   ps3_out_buf, buf_len);
 	iptc_data_free_buf (d, iptc_buf);
 	if (ps3_len < 0)
@@ -492,7 +401,7 @@ save_iptc_data (const char *filename,
 	outfile = fopen (tmpfile, "w");
 	if (!outfile)
 		goto abort4;
-	
+
 	if (iptc_jpeg_save_with_ps3 (infile, outfile, (guchar*)ps3_out_buf, ps3_len) < 0)
 		goto abort5;
 
@@ -528,32 +437,37 @@ abort1:
 
 
 static void
-save_comment_iptc (const char  *filename,
+save_comment_iptc (const char  *uri,
 		   CommentData *data)
 {
+        char        *local_file;
+        char        *local_uri;
+	time_t       mtime;
 	IptcData    *d;
 	IptcDataSet *ds;
-	time_t       mtime;
 	int          i;
 
-	mtime = get_file_mtime (filename);
+	local_file = get_cache_filename_from_uri (uri);
+        if (local_file == NULL)
+                return;
+	local_uri = get_uri_from_local_path (local_file);
+	mtime = get_file_mtime (local_uri);
 
-	d = iptc_data_new_from_jpeg (filename);
-	if (d) {
+	d = iptc_data_new_from_jpeg (local_file);
+	if (d != NULL) 
 		clear_iptc_comment (d);
-	}
 	else {
 		d = iptc_data_new ();
-		if (!d)
+		if (d == NULL)
 			return;
 	}
 
 	if (data->time > 0) {
 		struct tm t;
 		localtime_r (&data->time, &t);
-		
+
 		ds = iptc_dataset_new ();
-		if (ds) {
+		if (ds != NULL) {
 			iptc_dataset_set_tag (ds, IPTC_RECORD_APP_2,
 					IPTC_TAG_DATE_CREATED);
 			iptc_dataset_set_date (ds, t.tm_year + 1900, t.tm_mon + 1,
@@ -563,7 +477,7 @@ save_comment_iptc (const char  *filename,
 		}
 
 		ds = iptc_dataset_new ();
-		if (ds) {
+		if (ds != NULL) {
 			iptc_dataset_set_tag (ds, IPTC_RECORD_APP_2,
 					IPTC_TAG_TIME_CREATED);
 			iptc_dataset_set_time (ds, t.tm_hour, t.tm_min, t.tm_sec,
@@ -575,7 +489,7 @@ save_comment_iptc (const char  *filename,
 
 	for (i = 0; i < data->keywords_n; i++) {
 		ds = iptc_dataset_new ();
-		if (ds) {
+		if (ds != NULL) {
 			iptc_dataset_set_tag (ds, IPTC_RECORD_APP_2,
 					      IPTC_TAG_KEYWORDS);
 			iptc_dataset_set_data (ds, (guchar*)data->keywords[i],
@@ -587,7 +501,7 @@ save_comment_iptc (const char  *filename,
 
 	if (data->comment != NULL && data->comment[0]) {
 		ds = iptc_dataset_new ();
-		if (ds) {
+		if (ds != NULL) {
 			iptc_dataset_set_tag (ds, IPTC_RECORD_APP_2,
 					IPTC_TAG_CAPTION);
 			iptc_dataset_set_data (ds, (guchar*)data->comment,
@@ -599,7 +513,7 @@ save_comment_iptc (const char  *filename,
 
 	if (data->place != NULL && data->place[0]) {
 		ds = iptc_dataset_new ();
-		if (ds) {
+		if (ds != NULL) {
 			iptc_dataset_set_tag (ds, IPTC_RECORD_APP_2,
 					      IPTC_TAG_CONTENT_LOC_NAME);
 			iptc_dataset_set_data (ds, (guchar*)data->place,
@@ -609,34 +523,50 @@ save_comment_iptc (const char  *filename,
 		}
 	}
 
-	iptc_data_set_version (d, IPTC_IIM_VERSION);
-	iptc_data_set_encoding_utf8 (d);
+	/* The iptc_data_set_version and iptc_data_set_encoding_utf8
+	   functions cause Picasa to choke, as indicated in the libiptc
+	   API docs. So we'll disable them and see if that fixes more
+	   things than it breaks. Bug 438716. */
+
+	/* iptc_data_set_version (d, IPTC_IIM_VERSION); */
+	/* iptc_data_set_encoding_utf8 (d); */
+
 	iptc_data_sort (d);
 
-	save_iptc_data (filename, d);
-	set_file_mtime (filename, mtime);
-
+	save_iptc_data (local_file, d);
+	set_file_mtime (local_uri, mtime);
 	iptc_data_unref (d);
+
+        g_free (local_file);
+        g_free (local_uri);
 }
 
 
 static void
-delete_comment_iptc (const char  *filename)
+delete_comment_iptc (const char *uri)
 {
 	IptcData *d;
 	time_t    mtime;
+	char     *local_file;
 
-	mtime = get_file_mtime (filename);
-
-	d = iptc_data_new_from_jpeg (filename);
-	if (!d)
+	if ((uri == NULL) || ! is_local_file (uri))
 		return;
 
-	clear_iptc_comment (d);
-	save_iptc_data (filename, d);
-	set_file_mtime (filename, mtime);
+	mtime = get_file_mtime (uri);
 
+	local_file = get_cache_filename_from_uri (uri);
+	d = iptc_data_new_from_jpeg (local_file);
+	if (d == NULL) {
+		g_free (local_file);
+		return;
+	}
+
+	clear_iptc_comment (d);
+	save_iptc_data (local_file, d);
 	iptc_data_unref (d);
+	g_free (local_file);
+	
+	set_file_mtime (uri, mtime);
 }
 
 
@@ -650,14 +580,17 @@ comment_copy (const char *src,
 	char *comment_src = NULL;
 	char *comment_dest = NULL;
 
-	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
+	if (! is_local_file (src) || ! is_local_file (dest))
+		return;
+
+	comment_src = comments_get_comment_filename (src, TRUE);
 	if (! path_is_file (comment_src)) {
 		g_free (comment_src);
 		return;
 	}
 
-	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
-	if (path_is_file (comment_dest)) 
+	comment_dest = comments_get_comment_filename (dest, TRUE);
+	if (path_is_file (comment_dest))
 		file_unlink (comment_dest);
 
 	file_copy (comment_src, comment_dest);
@@ -674,14 +607,17 @@ comment_move (const char *src,
 	char *comment_src = NULL;
 	char *comment_dest = NULL;
 
-	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
+	if (! is_local_file (src) || ! is_local_file (dest))
+		return;
+
+	comment_src = comments_get_comment_filename (src, TRUE);
 	if (! path_is_file (comment_src)) {
 		g_free (comment_src);
 		return;
 	}
 
-	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
-	if (path_is_file (comment_dest)) 
+	comment_dest = comments_get_comment_filename (dest, TRUE);
+	if (path_is_file (comment_dest))
 		file_unlink (comment_dest);
 
 	file_move (comment_src, comment_dest);
@@ -692,17 +628,20 @@ comment_move (const char *src,
 
 
 void
-comment_delete (const char *filename)
+comment_delete (const char *uri)
 {
-	char *comment_name;
+	char *comment_uri;
 
-	comment_name = comments_get_comment_filename (filename, TRUE, TRUE);
-	file_unlink (comment_name);
-	g_free (comment_name);
+	if ((uri == NULL) || ! is_local_file (uri))
+		return;
+
+	comment_uri = comments_get_comment_filename (uri, TRUE);
+	file_unlink (comment_uri);
+	g_free (comment_uri);
 
 #ifdef HAVE_LIBIPTCDATA
-	if (image_is_jpeg (filename)) 
-		delete_comment_iptc (filename);
+	if (image_is_jpeg (uri))
+		delete_comment_iptc (uri);
 #endif /* HAVE_LIBIPTCDATA */
 }
 
@@ -714,7 +653,7 @@ get_utf8_text (CommentData *data,
 	if (value == NULL)
 		return NULL;
 
-	if (data->utf8_format) 
+	if (data->utf8_format)
 		return g_strdup (value);
 	else
 		return g_locale_to_utf8 (value, -1, NULL, NULL, NULL);
@@ -724,7 +663,7 @@ get_utf8_text (CommentData *data,
 static void
 get_keywords (CommentData *data,
 	      char        *utf8_value)
-              
+
 {
 	char     *value;
         int       n;
@@ -761,7 +700,7 @@ get_keywords (CommentData *data,
 	do {
 		gunichar ch = g_utf8_get_char (p);
 
-		done = (*p == 0); 
+		done = (*p == 0);
 
 		if ((ch == ',') || (*p == 0)) {
 			data->keywords[n++] = g_strndup (keyword, p - keyword);
@@ -775,92 +714,94 @@ get_keywords (CommentData *data,
 
 
 static CommentData *
-load_comment_from_xml (const char *filename)
+load_comment_from_xml (const char *uri)
 {
 	CommentData *data;
-	char        *comment_file;
+	char        *comment_uri;
+	char	    *local_file = NULL;
 	xmlDocPtr    doc;
         xmlNodePtr   root, node;
         xmlChar     *value;
 	xmlChar     *format;
 
-	if (filename == NULL)
+	if (uri == NULL)
 		return NULL;
 
-	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
-	if (! path_is_file (comment_file)) {
-		g_free (comment_file);
+	comment_uri = comments_get_comment_filename (uri, TRUE);
+	if (! path_exists (comment_uri)) {
+		g_free (comment_uri);
 		return NULL;
 	}
+	
+	local_file = get_cache_filename_from_uri (comment_uri);
+        doc = xmlParseFile (local_file);
 
-        doc = xmlParseFile (comment_file);
-	if (doc == NULL) {
-		g_free (comment_file);
+	g_free (comment_uri);
+        g_free (local_file);
+        
+	if (doc == NULL) 
 		return NULL;
-	}
-
+	
 	data = comment_data_new ();
-
         root = xmlDocGetRootElement (doc);
-        node = root->xmlChildrenNode;
-
 	format = xmlGetProp (root, FORMAT_TAG);
 	if (strcmp ((char*)format, "1.0") == 0)
 		data->utf8_format = FALSE;
 	else
 		data->utf8_format = TRUE;
-
-	for (; node; node = node->next) {
+	for (node = root->xmlChildrenNode; node; node = node->next) {
 		const char *name = (char*) node->name;
 
                 value = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 
-                if (strcmp (name, (char*)PLACE_TAG) == 0) 
+                if (strcmp (name, (char*)PLACE_TAG) == 0)
 			data->place = get_utf8_text (data, (char*)value);
-		else if (strcmp (name, (char*)NOTE_TAG) == 0) 
+		else if (strcmp (name, (char*)NOTE_TAG) == 0)
 			data->comment = get_utf8_text (data, (char*)value);
-		else if (strcmp (name, (char*)KEYWORDS_TAG) == 0) 
+		else if (strcmp (name, (char*)KEYWORDS_TAG) == 0)
 			get_keywords (data, (char*)value);
 		else if (strcmp (name, (char*)TIME_TAG) == 0) {
-			if (value != NULL) 
+			if (value != NULL)
 				data->time = atol ((char*)value);
 		}
-		
+
 		if (value)
 			xmlFree (value);
         }
 
 	xmlFree (format);
-
         xmlFreeDoc (doc);
-	g_free (comment_file);
 
-	return data;	
+	return data;
 }
 
 
 void
-save_comment (const char  *filename,
+save_comment (const char  *uri,
 	      CommentData *data,
 	      gboolean     save_embedded)
 {
 	xmlDocPtr    doc;
         xmlNodePtr   tree, subtree;
-	char        *comment_file = NULL;
+	char        *comment_uri = NULL;
+        char        *local_file = NULL;
 	char        *time_str = NULL;
 	char        *keywords_str = NULL;
 	char        *dest_dir = NULL;
 	char        *e_comment = NULL, *e_place = NULL, *e_keywords = NULL;
 
+	if ((data == NULL) || (uri == NULL) || ! is_local_file (uri))
+		return;
+
 	if (save_embedded) {
 #ifdef HAVE_LIBIPTCDATA
-		if (image_is_jpeg (filename)) 
-			save_comment_iptc (get_file_path_from_uri (filename), data);
+		if (image_is_jpeg (uri))
+			save_comment_iptc (uri, data);
 #endif /* HAVE_LIBIPTCDATA */
 	}
 
 	if (comment_data_is_void (data)) {
-		comment_delete (filename);
+		comment_delete (uri);
 		return;
 	}
 
@@ -872,18 +813,19 @@ save_comment (const char  *filename,
 			keywords_str = g_strdup (data->keywords[0]);
 		else
 			keywords_str = g_strjoinv (",", data->keywords);
-	} else
+	} 
+	else
 		keywords_str = g_strdup ("");
 
 	/* Escape text */
 
-	if (data->comment != NULL)
+	if (data->comment != NULL && g_utf8_validate (data->comment, -1, NULL))
 		e_comment = g_markup_escape_text (data->comment, -1);
 
-	if (data->place != NULL)
+	if (data->place != NULL && g_utf8_validate (data->place, -1, NULL))
 		e_place = g_markup_escape_text (data->place, -1);
 
-	if (keywords_str != NULL)
+	if (keywords_str != NULL && g_utf8_validate (keywords_str, -1, NULL))
 		e_keywords = g_markup_escape_text (keywords_str, -1);
 	g_free (keywords_str);
 
@@ -891,7 +833,7 @@ save_comment (const char  *filename,
 
 	doc = xmlNewDoc ((xmlChar*)"1.0");
 
-	doc->xmlRootNode = xmlNewDocNode (doc, NULL, COMMENT_TAG, NULL); 
+	doc->xmlRootNode = xmlNewDocNode (doc, NULL, COMMENT_TAG, NULL);
 	xmlSetProp (doc->xmlRootNode, FORMAT_TAG, FORMAT_VER);
 
 	tree = doc->xmlRootNode;
@@ -907,36 +849,37 @@ save_comment (const char  *filename,
 
 	/* Write to disk. */
 
-	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
-
-	dest_dir = remove_level_from_path (comment_file);
-
+	comment_uri = comments_get_comment_filename (uri, TRUE);
+        local_file = get_cache_filename_from_uri (comment_uri);
+	dest_dir = remove_level_from_path (comment_uri);
 	if (ensure_dir_exists (dest_dir, 0700)) {
 		xmlSetDocCompressMode (doc, 3);
-		xmlSaveFile (comment_file, doc);
+		xmlSaveFile (local_file, doc);
 	}
+	
 	g_free (dest_dir);
-	g_free (comment_file);
-
+	g_free (comment_uri);
+	g_free (local_file);
         xmlFreeDoc (doc);
 }
 
 
 CommentData *
-comments_load_comment (const char *filename,
+comments_load_comment (const char *uri,
 		       gboolean    try_embedded)
 {
-	CommentData *xml_comment = NULL, *img_comment = NULL;
+	CommentData *xml_comment = NULL;
+	CommentData *img_comment = NULL;
 
-	if (filename == NULL)
+	if ((uri == NULL) || ! is_local_file (uri))
 		return NULL;
 
-	xml_comment = load_comment_from_xml (filename);
+	xml_comment = load_comment_from_xml (uri);
 
 	if (try_embedded) {
 #ifdef HAVE_LIBIPTCDATA
-		if (image_is_jpeg (filename)) 
-			img_comment = load_comment_from_iptc (get_file_path_from_uri (filename));
+		if (image_is_jpeg (uri))
+			img_comment = load_comment_from_iptc (uri);
 		if (img_comment != NULL) {
 			if (xml_comment == NULL)
 				xml_comment = comment_data_new ();
@@ -947,13 +890,14 @@ comments_load_comment (const char *filename,
 #endif /* HAVE_LIBIPTCDATA */
 		if ((img_comment != NULL)
 		    && (! comment_data_equal (xml_comment, img_comment))) {
-			/* Consider the image comment more up-to-date and 
+			/* Consider the image comment more up-to-date and
 			 * sync the xml comment with it. */
-			save_comment (filename, img_comment, FALSE);
+			save_comment (uri, img_comment, FALSE);
 			comment_data_free (xml_comment);
 			xml_comment = img_comment;
 			xml_comment->changed = TRUE;
-		} else 
+		} 
+		else
 			comment_data_free (img_comment);
 	}
 
@@ -962,87 +906,97 @@ comments_load_comment (const char *filename,
 
 
 void
-comments_save_comment (const char  *filename,
+comments_save_comment (const char  *uri,
 		       CommentData *data)
 {
 	CommentData *new_data;
 
-	new_data = comments_load_comment (filename, FALSE);
+	if ((uri == NULL) || ! is_local_file (uri))
+		return;
 
+	new_data = comments_load_comment (uri, FALSE);
+	
 	if ((new_data == NULL) && (data != NULL)) {
 		CommentData *data_without_categories;
 
 		data_without_categories = comment_data_dup (data);
 		comment_data_free_keywords (data_without_categories);
-		save_comment (filename, data_without_categories, TRUE);
+		save_comment (uri, data_without_categories, TRUE);
 		comment_data_free (data_without_categories);
 
 		return;
 	}
+	
 	comment_data_free_comment (new_data);
 
 	if (data != NULL) {
-		if (data->place != NULL) 
+		if (data->place != NULL)
 			new_data->place = g_strdup (data->place);
 		if (data->time >= 0)
 			new_data->time = data->time;
-		if (data->comment != NULL) 
+		if (data->comment != NULL)
 			new_data->comment = g_strdup (data->comment);
 	}
 
-	save_comment (filename, new_data, TRUE);
+	save_comment (uri, new_data, TRUE);
 	comment_data_free (new_data);
 }
 
 
 void
-comments_save_comment_non_null (const char  *filename,
+comments_save_comment_non_null (const char  *uri,
 				CommentData *data)
 {
 	CommentData *new_data;
 
-	new_data = comments_load_comment (filename, TRUE);
+	if ((uri == NULL) || ! is_local_file (uri))
+		return;
+
+	new_data = comments_load_comment (uri, TRUE);
 	if (new_data == NULL) {
-		comments_save_comment (filename, data);
+		comments_save_comment (uri, data);
 		return;
 	}
 
 	/* Set non null fields. */
 
 	if (data->place != NULL) {
-		if (new_data->place != NULL) 
+		if (new_data->place != NULL)
 			g_free (new_data->place);
 		new_data->place = g_strdup (data->place);
 	}
-	
+
 	if (data->time >= 0)
 		new_data->time = data->time;
 
 	if (data->comment != NULL) {
-		if (new_data->comment != NULL) 
+		if (new_data->comment != NULL)
 			g_free (new_data->comment);
 		new_data->comment = g_strdup (data->comment);
 	}
 
-	comments_save_comment (filename, new_data);
+	comments_save_comment (uri, new_data);
 	comment_data_free (new_data);
 }
 
 
 void
-comments_save_categories (const char  *filename,
+comments_save_categories (const char  *uri,
 			  CommentData *data)
 {
 	CommentData *new_data;
 
-	new_data = comments_load_comment (filename, TRUE);
+	if ((uri == NULL) || ! is_local_file (uri))
+		return;
+
+	new_data = comments_load_comment (uri, TRUE);
 
 	if (new_data == NULL) {
 		CommentData *data_without_comment;
 
 		data_without_comment = comment_data_dup (data);
 		comment_data_free_comment (data_without_comment);
-		save_comment (filename, data_without_comment, TRUE);
+		save_comment (uri, data_without_comment, TRUE);
 		comment_data_free (data_without_comment);
 
 		return;
@@ -1062,7 +1016,7 @@ comments_save_categories (const char  *filename,
 		new_data->keywords[i] = NULL;
 	}
 
-	save_comment (filename, new_data, TRUE);
+	save_comment (uri, new_data, TRUE);
 	comment_data_free (new_data);
 }
 
@@ -1074,12 +1028,12 @@ comment_data_remove_keyword (CommentData *data,
 	gboolean  found = FALSE;
 	int       i;
 
-	if ((data->keywords == NULL) 
+	if ((data->keywords == NULL)
 	    || (data->keywords_n == 0)
 	    || (keyword == NULL))
 		return;
 
-	for (i = 0; i < data->keywords_n; i++) 
+	for (i = 0; i < data->keywords_n; i++)
 		if (g_utf8_collate (data->keywords[i], keyword) == 0) {
 			found = TRUE;
 			break;
@@ -1094,7 +1048,7 @@ comment_data_remove_keyword (CommentData *data,
 	data->keywords[i] = NULL;
 
 	data->keywords_n--;
-	data->keywords = g_realloc (data->keywords, 
+	data->keywords = g_realloc (data->keywords,
 				    sizeof (char*) * (data->keywords_n + 1));
 
 	if (data->keywords_n == 0) {
@@ -1113,12 +1067,12 @@ comment_data_add_keyword (CommentData *data,
 	if (keyword == NULL)
 		return;
 
- 	for (i = 0; i < data->keywords_n; i++) 
-		if (g_utf8_collate (data->keywords[i], keyword) == 0) 
+ 	for (i = 0; i < data->keywords_n; i++)
+		if (g_utf8_collate (data->keywords[i], keyword) == 0)
 			return;
-	
+
 	data->keywords_n++;
-	data->keywords = g_realloc (data->keywords, 
+	data->keywords = g_realloc (data->keywords,
 				    sizeof (char*) * (data->keywords_n + 1));
 	data->keywords[data->keywords_n - 1] = g_strdup (keyword);
 	data->keywords[data->keywords_n] = NULL;
@@ -1161,7 +1115,7 @@ comment_text_is_void (CommentData *data)
 }
 
 
-/* based on glib/glib/gmarkup.c (Copyright 2000, 2003 Red Hat, Inc.)  
+/* based on glib/glib/gmarkup.c (Copyright 2000, 2003 Red Hat, Inc.)
  * This version does not escape ' and ''. Needed  because IE does not recognize
  * &apos; and &quot; */
 static void
@@ -1176,53 +1130,53 @@ _append_escaped_text_for_html (GString     *str,
 
 	p = text;
 	end = text + length;
-	
+
 	while (p != end) {
 		const gchar *next;
 		next = g_utf8_next_char (p);
 		ch = g_utf8_get_char (p);
-		
+
 		switch (state) {
 		    case 1: /* escaped */
-			if ((ch > 127) ||  !isprint((char)ch)) 
+			if ((ch > 127) ||  !isprint((char)ch))
 				g_string_append_printf (str, "\\&#%d;", ch);
 			else
-				g_string_append_unichar (str, ch);			
+				g_string_append_unichar (str, ch);
 			state = 0;
 			break;
-			
+
 		    default: /* not escaped */
 			switch (*p) {
 			    case '\\':
 				state = 1; /* next character is escaped */
 				break;
-			
+
 			    case '&':
 				g_string_append (str, "&amp;");
 				break;
-			
+
 			    case '<':
 				g_string_append (str, "&lt;");
 				break;
-			
+
 			    case '>':
 				g_string_append (str, "&gt;");
 				break;
-			
+
 			    case '\n':
 				g_string_append (str, "<br />");
 				break;
 
 			    default:
-				if ((ch > 127) ||  !isprint((char)ch)) 
+				if ((ch > 127) ||  !isprint((char)ch))
 					g_string_append_printf (str, "&#%d;", ch);
 				else
-					g_string_append_unichar (str, ch);			
+					g_string_append_unichar (str, ch);
 				state = 0;
 				break;
 			}
 		}
-		
+
 		p = next;
 	}
 }
@@ -1235,7 +1189,7 @@ _g_escape_text_for_html (const gchar *text,
 	GString *str;
 
 	g_return_val_if_fail (text != NULL, NULL);
-	
+
 	if (length < 0)
 		length = strlen (text);
 
@@ -1250,22 +1204,22 @@ _g_escape_text_for_html (const gchar *text,
 static void
 _string_append (GString    *str,
 		const char *a,
-		gboolean    markup_escape) 
+		gboolean    markup_escape)
 {
 	if (a == NULL)
 		return;
 
  	if (markup_escape)
 		_append_escaped_text_for_html (str, a, strlen (a));
-	else 
+	else
 		g_string_append (str, a);
 }
 
 
 /* Note: separators are not escaped */
 static char *
-_get_comment_as_string_common (CommentData *data, 
-			       char        *sep1, 
+_get_comment_as_string_common (CommentData *data,
+			       char        *sep1,
 			       char        *sep2,
 			       gboolean     markup_escape)
 {
@@ -1276,7 +1230,7 @@ _get_comment_as_string_common (CommentData *data,
 
 	if (data == NULL)
 		return NULL;
- 
+
 	if (data->time != 0) {
 		tm = localtime (& data->time);
 		if (tm->tm_hour + tm->tm_min + tm->tm_sec == 0)
@@ -1284,52 +1238,52 @@ _get_comment_as_string_common (CommentData *data,
 		else
 			strftime (time_txt, 50, _("%d %B %Y, %H:%M"), tm);
 		utf8_time_txt = g_locale_to_utf8 (time_txt, -1, 0, 0, 0);
-	} 
+	}
 
-	if ((data->comment == NULL) 
-	    && (data->place == NULL) 
+	if ((data->comment == NULL)
+	    && (data->place == NULL)
 	    && (data->time == 0)) {
-		if (data->keywords_n > 0) 
+		if (data->keywords_n > 0)
 			as_string = NULL;
 		else if (markup_escape)
 			as_string = g_markup_escape_text (_("(No Comment)"), -1);
 		else
 			as_string = g_strdup (_("(No Comment)"));
-		
+
 	} else {
 		GString *comment;
 
 		comment = g_string_new ("");
 
-		if (data->comment != NULL) 
+		if (data->comment != NULL)
 			_string_append (comment, data->comment, markup_escape);
 
 		if ((data->comment != NULL)
 		    && ((data->place != NULL) || (*time_txt != 0)))
 			g_string_append (comment, sep1);
-		
-		if (data->place != NULL) 
+
+		if (data->place != NULL)
 			_string_append (comment, data->place, markup_escape);
 
 		if ((data->place != NULL) && (*time_txt != 0))
 			g_string_append (comment, sep2);
-		
-		if (utf8_time_txt != NULL) 
+
+		if (utf8_time_txt != NULL)
 			_string_append (comment, utf8_time_txt, markup_escape);
-		
+
 		as_string = comment->str;
 		g_string_free (comment, FALSE);
 	}
-	
+
 	g_free (utf8_time_txt);
-	
+
 	return as_string;
 }
 
 
 char *
-comments_get_comment_as_string (CommentData *data, 
-				char        *sep1, 
+comments_get_comment_as_string (CommentData *data,
+				char        *sep1,
 				char        *sep2)
 {
 	return _get_comment_as_string_common (data, sep1, sep2, FALSE);
@@ -1337,8 +1291,8 @@ comments_get_comment_as_string (CommentData *data,
 
 
 char *
-comments_get_comment_as_xml_string (CommentData *data, 
-				    char        *sep1, 
+comments_get_comment_as_xml_string (CommentData *data,
+				    char        *sep1,
 				    char        *sep2)
 {
 	return _get_comment_as_string_common (data, sep1, sep2, TRUE);
