@@ -222,11 +222,12 @@ gth_location_open_other (GthLocation *location)
 					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 					       NULL);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), FALSE);
-	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
-	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (chooser), gth_location_get_folder_uri (location));
 
-	g_signal_connect (G_OBJECT (GTK_DIALOG (chooser)),
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), FALSE);
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (chooser), gth_location_get_folder_uri (location));
+	gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_OK);
+
+	g_signal_connect (G_OBJECT (chooser),
 			  "response",
 			  G_CALLBACK (open_other_location_response_cb),
 			  location);
@@ -256,6 +257,7 @@ combo_changed_cb (GtkComboBox *widget,
 	GthLocation *loc = user_data;
 	GtkTreeIter  iter;
 	char        *path = NULL;
+	char        *uri = NULL;
 	int          item_type = ITEM_TYPE_NONE;
 
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (loc->priv->combo), &iter))
@@ -265,6 +267,7 @@ combo_changed_cb (GtkComboBox *widget,
 			    &iter,
 			    PATH_COLUMN, &path,
 			    TYPE_COLUMN, &item_type,
+			    PATH_COLUMN, &uri,
 			    -1);
 
 	if (item_type == ITEM_TYPE_OPEN_LOCATION) {
@@ -278,8 +281,12 @@ combo_changed_cb (GtkComboBox *widget,
 	if (path == NULL)
 		return;
 
+	g_free (loc->priv->uri);
+	loc->priv->uri = add_scheme_if_absent (uri);
+
 	g_signal_emit (G_OBJECT (loc), gth_location_signals[CHANGED], 0, path);
 	g_free (path);
+	g_free (uri);
 }
 
 
